# Copyrights 2009-2015 by [Mark Overmeer].
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 2.01.
use warnings;
use strict;

package XML::Compile::SOAP12::Client;
use vars '$VERSION';
$VERSION = '3.03';

use base 'XML::Compile::SOAP12', 'XML::Compile::SOAP::Client';

use Log::Report 'xml-compile-soap12', syntax => 'SHORT';


# much work to do ;-)

1;
